import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:yflutter_widgets/configs/screenutil_creator.dart';
import 'package:yflutter_widgets/widgets/listview/headerfooter/custom_refresh_footer.dart';
import 'package:yflutter_widgets/widgets/listview/headerfooter/custom_refresh_header.dart';
import 'package:yflutter_widgets/widgets/listview/model/ylvsectionmodel.dart';
import 'package:yflutter_widgets/widgets/listview/ylistviewbody.dart';
import 'package:yflutter_widgets/widgets/listview/ylvconfigs.dart';
import 'package:yflutter_widgets/widgets/listview/ylvheaders/ylvappbarheader.dart';
import 'package:yflutter_widgets/widgets/listview/ylvheaders/ylvzoomheader.dart';
import 'package:yflutter_widgets/widgets/listview/ylvproviders/contractedbarprovider.dart';
import 'package:yflutter_widgets/widgets/ycustom_card.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:extended_nested_scroll_view/extended_nested_scroll_view.dart'
    as extended;

/// ListView Header Type
/// None: No header, as Default,
/// AppBar: Provide a appbar level header with contracted/zoom feature,
/// * Do not set AppBar in your view/page when use this type, cuz it already provides,
/// * set height of it,
/// Zoom: A Header with contracted/zoom feature, under app bar, set height(max) and min height,
/// Normal: A scrollable header, which means will be out of view when scroll, set height only, default 150.
enum HeaderType {
  None,
  AppBar,
  Zoom,
  Normal,
}

/// Boarder of ListView by YCustomCard style
/// None: no boarder, as Default,
/// ListWhole, set boarder for whole ListView including Header,
/// ListSection, set boarder for each section.
///
/// TODO: Next Step, add boarder for whole ListView except Header.
enum BorderType {
  None,
  ListWhole,
  ListSection,
}

typedef ZoomHeaderContractedCallback = void Function(bool isContracted);
typedef ZoomHeaderScrollCallback = void Function(double offset);

class YListView extends StatefulWidget {
  YListView(
      {this.type = HeaderType.None,
      this.borderType = BorderType.None,
      this.borderMargin =
          const EdgeInsets.only(left: 16, top: 16, right: 16, bottom: 16),
      this.title = "",
      this.titleColor = Colors.black,
      this.contractedTitleColor = Colors.white,
      this.headerColor = Colors.white,
      this.elevation = 10,
      this.isForcedElevated = false,
      this.listHeader,
      this.headerHeight = 150.0,
      this.headerMinHeight = 44.0,
      @required this.numOfSection,
      @required this.numberOfRowsInSection,
      @required this.sectionHeight,
      @required this.cellHeight,
      @required this.sectionHeader,
      @required this.cell,
      this.sectionHeadersPinToVisibleBounds = false,
      this.contractedCallback,
      this.scrollCallback,
      this.leftBarItem,
      this.rightBarItems,
      this.hasRefresh = false,
      this.refreshHeader,
      this.refreshFooter,
      this.refreshCallback,
      this.loadCallback,
      this.refreshController});

  /*
  * Type Setting.
  * */
  // Type of the Header, more details in HeaderType.
  final HeaderType type;

  // Type of the Border, more details in BorderType.
  final BorderType borderType;

  // Border EdgeInsets.
  final EdgeInsets borderMargin;

  /*
  * Header Setting.
  * */
  // Title of the AppBar Type Header.
  final String title;

  // The color of the Title when expanded.
  final Color titleColor;

  // The color of the Title when contracted.
  final Color contractedTitleColor;

  // The background color of Header.
  final Color headerColor;

  // Set the elevation of the Appbar Shadow, default 10.
  final double elevation;

  // Is Forced to elevated, default false.
  final bool isForcedElevated;

  // Left AppBar Item, normally back.
  final Widget leftBarItem;

  // Right AppBar Items, List.
  final List<Widget> rightBarItems;

  // Header Content for ListView Header when HeaderType is not None.
  final Widget listHeader;

  // Max/Normal Height of the Header.
  final double headerHeight;

  // Min Height of the Header, only when it can be contracted.
  final double headerMinHeight;

  /*
  * List Body/Content Setting.
  * */
  // Number of Section for the list.
  final int numOfSection;

  // Number of Rows/Cells in each section.
  final NumberOfRowsInSection numberOfRowsInSection;

  // Section Height.
  final SectionHeight sectionHeight;

  // Row/Cell Height.
  final CellHeight cellHeight;

  // Section Header for each section, with Section Index, see above def..
  final HeaderOfSectionAtIndex sectionHeader;

  // Row/Cell widget for each row, with related Section Index and Row Index, see above def..
  final CellOfRowAtIndex cell;

  // Section Header Pin feature, default false.
  final bool sectionHeadersPinToVisibleBounds;

  // Refresh Setting.
  final bool hasRefresh;
  // Refresh Header Widget, see default below.
  final Header refreshHeader;
  // Refresh Footer Widget, see default below.
  final Footer refreshFooter;
  // Refresh Controller.
  final EasyRefreshController refreshController;

  /*
  * Callback Setting.
  * */
  // Callback when the Header remains only AppBar height.
  final ZoomHeaderContractedCallback contractedCallback;
  // Callback for the Header scroll offset.
  final ZoomHeaderScrollCallback scrollCallback;

  final OnRefreshCallback refreshCallback;
  final OnLoadCallback loadCallback;

  @override
  _YListViewState createState() => _YListViewState();
}

class _YListViewState extends State<YListView> {
  ScrollController _scrollController;
  ContractedBarProvider _contractedBarProvider = ContractedBarProvider();
  bool lastContractedStatus = false;

  List<YLVSectionModel> sections = [];
  List<Widget> total = [];

  // EasyRefreshController _refreshController = new EasyRefreshController();

  bool get isContracted {
    return _scrollController.hasClients &&
        // _scrollController.offset > (180 - kToolbarHeight);
        _scrollController.offset > kToolbarHeight;
  }

  // Not use now.
  _scrollListener() {
    widget.scrollCallback.call(_scrollController.offset);
    if (lastContractedStatus != isContracted) {
      lastContractedStatus = isContracted;
      _contractedBarProvider.contractStatusChange(isContracted);
      widget.contractedCallback.call(isContracted);
    }
  }

  @override
  void initState() {
    _scrollController = ScrollController();
    _scrollController.addListener(_scrollListener);
    initBaseData();
    super.initState();
  }

  @override
  void dispose() {
    _scrollController.removeListener(_scrollListener);
    _scrollController.dispose();
    _contractedBarProvider.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    ScreenUtilCreator.init(context);
    return ChangeNotifierProvider<ContractedBarProvider>.value(
      value: _contractedBarProvider,
      child: extended.NestedScrollView(
        headerSliverBuilder: (context, innerBoxIsScrolled) {
          return [
            // if (widget.type != HeaderType.None)
              // SliverOverlapAbsorber(
              //   handle:
              //       NestedScrollView.sliverOverlapAbsorberHandleFor(context),
              //   sliver:
              zoomHeaderConfig(),
            // ),
          ];
        },
        body: widget.hasRefresh?
        EasyRefresh.builder(
          enableControlFinishRefresh: true,
          enableControlFinishLoad: true,
          header: widget.refreshHeader==null?CustomRefreshHeader(
              extent: 60,
              triggerDistance: 70,
              refreshingText: "加载中...",
              refreshReadyText: "释放即可刷新",
              textColor: Colors.grey,
              refreshText: "下拉即可刷新",
              refreshedText: "刷新完成",
              showInfo: false):widget.refreshHeader,
          footer: widget.refreshFooter==null?CustomRefreshFooter(
              loadingText: '加载中...',
              loadReadyText: '向上拉动查看更多',
              enableHapticFeedback: false,
              loadedText: '没有更多了哦',
              noMoreText: '没有更多了哦',
              textColor: Colors.grey,
              showInfo: false):widget.refreshFooter,
          controller: widget.refreshController,
          builder: (context, physics, header, footer) {
            return contentConfig(physics: physics, header: header, footer: footer);
          },
          onRefresh: widget.refreshCallback,
          onLoad: widget.loadCallback,
        )
        : contentConfig(),
      ),
    );
  }

  /// Setup ListVie Header
  Widget zoomHeaderConfig() {
    // AppBar Header
    if (widget.type == HeaderType.AppBar)
      return YLVAppBarHeader(
        title: widget.title,
        contractedTitleColor: widget.contractedTitleColor,
        headerColor: widget.headerColor,
        titleColor: widget.titleColor,
        leftBarItem: widget.leftBarItem,
        rightBarItems: widget.rightBarItems,
        headerHeight: widget.headerHeight,
        elevation: widget.elevation,
        isForcedElevated: widget.isForcedElevated,
        insideWidget: Row(
          children: [Expanded(child: widget.listHeader)],
        ),
      );
    // Zoom Header
    if (widget.type == HeaderType.Zoom)
      return YLVZoomHeader(
        insideWidget: widget.listHeader,
        minHeight: widget.headerMinHeight,
        maxHeight: widget.headerHeight,
        pinned: true,
      );
    // Normal Header
    if (widget.type == HeaderType.Normal)
      return YLVZoomHeader(
        insideWidget: widget.listHeader,
        minHeight: widget.headerHeight,
        maxHeight: widget.headerHeight,
        pinned: false,
      );
    else
      return YLVZoomHeader(
        insideWidget: Container(),
        minHeight: 0,
        maxHeight: 1,
        pinned: false,
      );
  }

  Widget contentConfig({ScrollPhysics physics, Widget header, Widget footer}) {
    return widget.borderType == BorderType.ListWhole
        ? YCustomCard(
            insideWidget: detailContentConfig(physics: physics, header: header, footer: footer),
            sEdgeInsets: widget.borderMargin)
        : detailContentConfig(physics: physics, header: header, footer: footer);
  }

  Widget detailContentConfig(
  {ScrollPhysics physics, Widget header, Widget footer}) {
    return CustomScrollView(
      // controller: _scrollController,
      physics: physics,
      slivers: [
        if (widget.hasRefresh) header,
        //   // if (widget.type != HeaderType.None)
        //   //   SliverOverlapInjector(
        //   //     // This is the flip side of the SliverOverlapAbsorber
        //   //     // above.
        //   //     handle: NestedScrollView
        //   //         .sliverOverlapAbsorberHandleFor(context),
        //   //   ),
        ...total,
        if (widget.hasRefresh) footer,
      ],
    );
  }

  /// Setup ListView Content
  void initBaseData() {
    this.sections.clear();
    this.total.clear();

    for (int sectionIndex = 0;
        sectionIndex < widget.numOfSection;
        sectionIndex++) {
      int rowCount = widget.numberOfRowsInSection(sectionIndex);
      Widget sectionHeader = widget.sectionHeader(context, sectionIndex);
      double sectionHeight;
      if (sectionHeader != null) {
        sectionHeight = this.widget.sectionHeight(context, sectionIndex);
      } else {
        sectionHeight = 0;
      }
      if (!widget.sectionHeadersPinToVisibleBounds) {
        sectionHeader = SliverToBoxAdapter(
            child: widget.sectionHeader(context, sectionIndex));
      } else {
        sectionHeader = YLVZoomHeader(
          insideWidget: widget.sectionHeader(context, sectionIndex),
          minHeight: sectionHeight,
          maxHeight: sectionHeight,
        );
      }

      YLVSectionModel model = YLVSectionModel(
        height: sectionHeight,
        section: sectionIndex,
        headerWidget: sectionHeader,
      );
      sections.add(model);

      // Add Header for each section
      total.add(sectionHeader);
      // Add Section Body
      total.add(YLVBody(
        cell: widget.cell,
        numRow: rowCount,
        sectionIndex: sectionIndex,
        hasBoarder: widget.borderType == BorderType.ListSection,
      ));
    }
  }
}
